// --------------------------------------------------------------------
//        G e t o p t . c p p
//
//        Fido messages tracker
//        AT&T public domain source for getopt(3)
// --------------------------------------------------------------------
//        Copyright (c) 1998-2000 by Fyodor Ustinov
//                                FIDONet 2:5020/79
//        Copyright (c) 2004      by Alex Soukhotine
//                                FIDONet 2:5030/1157
//
//        All rights reserved.
// --------------------------------------------------------------------
// $Id: getopt.cpp,v 1.6 2004/09/12 16:14:47 ph0enix Exp $
// --------------------------------------------------------------------
/*
   01 Oct 89   Added function prototype for getopt()              ahd
   22 Dec 91   Use Mitch Mitchell's fixes for - option            ahd
*/

/*
 * Here's something you've all been waiting for:  the AT&T public domain
 * source for getopt(3).  It is the code which was given out at the 1985
 * UNIFORUM conference in Dallas.  I obtained it by electronic mail
 * directly from AT&T.  The people there assure me that it is indeed
 * in the public domain.
 *
 * There is no manual page.  That is because the one they gave out at
 * UNIFORUM was slightly different from the current System V Release 2
 * manual page.  The difference apparently involved a note about the
 * famous rules 5 and 6, recommending using white space between an option
 * and its first argument, and not grouping options that have arguments.
 * Getopt itself is currently lenient about both of these things White
 * space is allowed, but not mandatory, and the last option in a group can
 * have an argument.  That particular version of the man page evidently
 * has no official existence, and my source at AT&T did not send a copy.
 * The current SVR2 man page reflects the actual behavor of this getopt.
 * However, I am not about to post a copy of anything licensed by AT&T.
 */
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "getopt.hpp"

static int   optionInd = 1;
static int   optionOpt;
char  *optionArg;

int ParseCmdArgs(int argc, char **argv, char *opts)
{
   static int sp = 1;
   int c;
   char *cp;

   if (optionInd < argc && argv[optionInd][0] == '-' && argv[optionInd][1] == '\0')
       if((cp=strchr(opts, '-')) != NULL) {
           optionInd++;
           return('-');
       } else {
           optionInd++;
           return('?');
       }

   if(sp == 1)
      if(optionInd >= argc || argv[optionInd][0] != '-' || argv[optionInd][1] == '\0')
         return(EOF);
      else if(strcmp(argv[optionInd], "--") == 0) {
         optionInd++;
         return(EOF);
      }

   optionOpt = c = argv[optionInd][sp];

   if(c == ':' || (cp=strchr(opts, c)) == NULL) {
      fprintf(stderr,"%s: illegal option -- %c\n", argv[0], c);
      if(argv[optionInd][++sp] == '\0') {
         optionInd++;
         sp = 1;
      }
      return('?');
   }

   if(*++cp == ':') {
      if(argv[optionInd][sp+1] != '\0')
         optionArg = &argv[optionInd++][sp+1];
      else if(++optionInd >= argc) {
         fprintf(stderr,"%s: option requires an argument -- %c\n", argv[0], c);
         sp = 1;
         return('?');
      } else
         optionArg = argv[optionInd++];
      sp = 1;
   } else {
      if(argv[optionInd][++sp] == '\0') {
         sp = 1;
         optionInd++;
      }
      optionArg = NULL;
   }
   return(c);
} /* getopt */
